package //݂Ȃ̃pbP[WLĂ

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import android.opengl.GLES20;

public class MySquare {
	
	private FloatBuffer mVertexBuffer;
	private ShortBuffer mDrawOrderBuffer;

	private final float[] mVertexData = { 
		-0.6f, 0.6f, 0.0f, 0.0f, 0.6f, 0.6f, 1.0f, //0ɐ
		-0.6f, -0.6f, 0.0f, 0.0f, 0.6f, 0.6f, 1.0f, //1ɐ
		0.6f, -0.6f, 0.0f, 0.0f, 1.0f, 0.0f, 1.0f, //2ɗ
		0.6f, 0.6f, 0.0f, 0.0f, 1.0f, 0.0f, 1.0f, //3ɗ
	};

	private short mDrawOrder[] ={0, 1, 2, 0, 2, 3};


	private static final int FLOAT_SIZE_BYTES = 4;
	private static final int SHORT_SIZE_BYTES = 2;

	private static final int XYZ_DATA_OFFSET =3;
	private static final int COLOR_DATA_OFFSET =4;
	private static final int XYZ_AND_COLOR_STRIDE = 7;
	
	private int mProgram;
	private int maPositionHandle;
	private int maColorHandle;
	private int muMVPMatrixHandle;
	
	public MySquare(){
		
		mVertexBuffer = allocateFloatBuffer(mVertexData.length);
		mVertexBuffer.put(mVertexData).position(0);
		
		mDrawOrderBuffer = allocateShortBuffer(mDrawOrder.length);
        mDrawOrderBuffer.put(mDrawOrder).position(0);

		mProgram = createProgram(vertexShaderCode, fragmentShaderCode);
		maPositionHandle = GLES20.glGetAttribLocation(mProgram, "aPosition");
		maColorHandle = 
				GLES20.glGetAttribLocation(mProgram, "aColor"); 
		muMVPMatrixHandle = GLES20.glGetUniformLocation(
				mProgram, "uMVPMatrix");
	
	}
	
	public void draw(float[] mvpMatrix){
		
        GLES20.glUseProgram(mProgram);
		
		mVertexBuffer.position(0);
		GLES20.glEnableVertexAttribArray(maPositionHandle);
		GLES20.glVertexAttribPointer(
		  maPositionHandle, XYZ_DATA_OFFSET, GLES20.GL_FLOAT, false,
				XYZ_AND_COLOR_STRIDE*FLOAT_SIZE_BYTES,mVertexBuffer); 

		mVertexBuffer.position(XYZ_DATA_OFFSET);
		GLES20.glEnableVertexAttribArray(maColorHandle);
		GLES20.glVertexAttribPointer(
		  maColorHandle, COLOR_DATA_OFFSET, GLES20.GL_FLOAT, false,
				XYZ_AND_COLOR_STRIDE*FLOAT_SIZE_BYTES,mVertexBuffer);
		
		GLES20.glUniformMatrix4fv(
				muMVPMatrixHandle, 1, false, mvpMatrix, 0 );

		GLES20.glDrawElements(
				GLES20.GL_TRIANGLES,
				mDrawOrder.length,
				GLES20.GL_UNSIGNED_SHORT,
				mDrawOrderBuffer
		);

	
		GLES20.glDisableVertexAttribArray(maPositionHandle);
		GLES20.glDisableVertexAttribArray(maColorHandle); 
		
	}
	
	private FloatBuffer allocateFloatBuffer(int dataLength) {
		return ByteBuffer.allocateDirect(dataLength * FLOAT_SIZE_BYTES)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();

	}
	
	private ShortBuffer allocateShortBuffer(int dataLength) {
		return ByteBuffer.allocateDirect(dataLength * SHORT_SIZE_BYTES)
				.order(ByteOrder.nativeOrder()).asShortBuffer();
	}

	private int loadShader(int shaderType, String source) {
		int shader = GLES20.glCreateShader(shaderType);
		GLES20.glShaderSource(shader, source);
		GLES20.glCompileShader(shader);

		return shader;
	}

	private int createProgram(String vertexSource, String fragmentSource) {

		int vertexShader = loadShader(GLES20.GL_VERTEX_SHADER, vertexSource);
		int fragmentShader = loadShader(GLES20.GL_FRAGMENT_SHADER,
				fragmentSource);

		int program = GLES20.glCreateProgram();

		GLES20.glAttachShader(program, vertexShader);
		GLES20.glAttachShader(program, fragmentShader);

		GLES20.glLinkProgram(program);

		return program;

	}

	
	private final String vertexShaderCode = 
		"uniform mat4 uMVPMatrix;" +
		"attribute vec4 aPosition;" +
		"attribute vec4 aColor;"+
		"varying vec4 vColor;"+ 
		"void main() {" +
		"  gl_Position = uMVPMatrix*aPosition;"+
		"  vColor = aColor;"+
		"}";

	private final String fragmentShaderCode =
	    "precision lowp float;" +
	    "varying vec4 vColor;"+ 
		"void main() {" +
		"  gl_FragColor = vColor;" +
		"}";

}
