package //݂Ȃ̃pbP[WLĂ

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import android.opengl.GLES20;

public class MyTriangle {
	
	private FloatBuffer mVertexBuffer;

	private final float[] mVertexData = { 
		0.0f,  0.52f, 0.0f, 0.5f, -2.0f, //^̒_
	   -0.6f, -0.52f, 0.0f, -1.5f, 1.5f, //̒_
		0.6f, -0.52f, 0.0f, 2.0f, 1.5f  //E̒_
    };

	private static final int FLOAT_SIZE_BYTES = 4;
	private static final int XYZ_DATA_OFFSET =3;
	private static final int TEX_XY_OFFSET=2;
	private static final int VERTEX_AND_TEX_STRIDE=5;
	
	private static final int NUMBER_OF_VERTICES =3;

	private int mProgram;
	private int maPositionHandle;
	private int maTexHandle;
	private int muMVPMatrixHandle;
	
	public MyTriangle(){
		
		mVertexBuffer = allocateFloatBuffer(mVertexData.length);
		mVertexBuffer.put(mVertexData).position(0);
		
		mProgram = createProgram(vertexShaderCode, fragmentShaderCode);
		maPositionHandle = GLES20.glGetAttribLocation(mProgram, "aPosition");
		maTexHandle = GLES20.glGetAttribLocation(mProgram, "aTexture");
		muMVPMatrixHandle = GLES20.glGetUniformLocation(
				mProgram, "uMVPMatrix");
	
	}
	
	public void draw(float[] mvpMatrix){
		
        GLES20.glUseProgram(mProgram);
		
        mVertexBuffer.position(0); 
		GLES20.glEnableVertexAttribArray(maPositionHandle);
		GLES20.glVertexAttribPointer(maPositionHandle, XYZ_DATA_OFFSET, GLES20.GL_FLOAT,  
				false, VERTEX_AND_TEX_STRIDE*FLOAT_SIZE_BYTES, mVertexBuffer);  
		
		
		mVertexBuffer.position(XYZ_DATA_OFFSET);
		
		GLES20.glEnableVertexAttribArray(maTexHandle);
		GLES20.glVertexAttribPointer(maTexHandle, TEX_XY_OFFSET, GLES20.GL_FLOAT,
				false, VERTEX_AND_TEX_STRIDE*FLOAT_SIZE_BYTES, mVertexBuffer);
		
		GLES20.glUniformMatrix4fv(
				muMVPMatrixHandle, 1, false, mvpMatrix, 0 );

		GLES20.glDrawArrays(
				GLES20.GL_TRIANGLES, 0, NUMBER_OF_VERTICES);
	
		GLES20.glDisableVertexAttribArray(maPositionHandle);
		GLES20.glDisableVertexAttribArray(maTexHandle); 
		
	}
	
	private FloatBuffer allocateFloatBuffer(int dataLength) {
		return ByteBuffer.allocateDirect(dataLength * FLOAT_SIZE_BYTES)
				.order(ByteOrder.nativeOrder()).asFloatBuffer();

	}

	private int loadShader(int shaderType, String source) {
		int shader = GLES20.glCreateShader(shaderType);
		GLES20.glShaderSource(shader, source);
		GLES20.glCompileShader(shader);

		return shader;
	}

	private int createProgram(String vertexSource, String fragmentSource) {

		int vertexShader = loadShader(GLES20.GL_VERTEX_SHADER, vertexSource);
		int fragmentShader = loadShader(GLES20.GL_FRAGMENT_SHADER,
				fragmentSource);

		int program = GLES20.glCreateProgram();

		GLES20.glAttachShader(program, vertexShader);
		GLES20.glAttachShader(program, fragmentShader);

		GLES20.glLinkProgram(program);

		return program;

	}

	
	private final String vertexShaderCode = 
		"uniform mat4 uMVPMatrix;" +
		"attribute vec4 aPosition;" +
		"attribute vec2 aTexture;"+
		"varying vec2 vTexture;"+
		"void main() {" +
		"  gl_Position = uMVPMatrix*aPosition;"+
		"  vTexture = aTexture;"+
		"}";



	private final String fragmentShaderCode =
			"precision lowp float;" +
			"varying vec2 vTexture;"+
			"uniform sampler2D sTexture;"+
			"void main() {" +
			"  gl_FragColor = texture2D(sTexture, vTexture);" +
			"}";


}
